require 'httparty'
require 'nokogiri'
require 'logger'

module TMAPI
  # TaxiMaster Telephony API
  class Telephony
    class Error < RuntimeError; end
    class OrderNotFound < Error; end
    class FailedRequirements < Error; end

    def initialize(url, key, opts = {})
      @url = url
      @key = key
      @verify_ssl = opts[:ssl] || true
      @logger = TMAPI::Telephony.logger || Logger.new(STDOUT)
      @debug = opts[:debug] || false
    end

    def change_order_state(order_id, state)
      params = { order_id: order_id, need_state: state }
      params[:signature] = signature(params)

      opts = { body: params, verify: @verify_ssl, format: :plain }
      response = HTTParty.post("#{@url}/change_order_state", opts)
      @logger.debug(response) if @debug

      parse_change_order_state(response, order_id)
    end

    private

    class << self
      attr_accessor :logger
    end

    if defined?(Rails::Railtie)
      # Use Rails.logger inside Rails app
      class Railtie < Rails::Railtie
        initializer 'TMAPI::Telephony.logger' do
          TMAPI::Telephony.logger = Rails.logger
        end
      end
    end

    def url_params(params)
      URI.encode_www_form(params)
    end

    def signature(params)
      Digest::MD5.hexdigest(url_params(params) + @key)
    end

    def inspect(data)
      data.each_with_object({}) do |elem, hash|
        hash[elem.name.downcase.to_sym] = if elem.elements.any?
                                            inspect(elem.elements)
                                          else
                                            elem.text
                                          end
      end
    end

    def parse_change_order_state(response, order_id)
      data = Nokogiri::XML(response.body).root
      return {} unless data

      parsed_response = inspect(data.elements)
      case parsed_response[:code]
      when '100'
        raise OrderNotFound.new, "Order ID #{order_id} not found"
      when '101'
        raise FailedRequirements.new,
              "Can't change state of order ID #{order_id} due to requirements"
      end

      parsed_response[:data]
    end
  end
end
